package agents;

import enums.Phases;
import enums.Types;
import estructures.MapPosition;

import java.util.Random;

import managers.MapManager;
import behaviours.*;
import jade.core.Agent;
import jade.core.behaviours.*;

public class HerbivoreAgent extends LivingAgent {

	public HerbivoreAgent(){
		type=Types.HERBIVORE;
		phase=Phases.YOUNG;
		
		watchingDistance = 6;
		borningDistance = 3;
		reproductionEnergy = 50;
		eatingEnergy = 200;
		energy = 100 + (int) (50 * new Random().nextFloat());
		maxEnergy = 400;
	}
	
	
	protected void setup() {
		super.setup();

		Behaviour behaviour;
		SequentialBehaviour seq = new SequentialBehaviour(this);
		ParallelBehaviour par = new ParallelBehaviour(this,
				ParallelBehaviour.WHEN_ANY);

		par = new ParallelBehaviour(this, ParallelBehaviour.WHEN_ANY);
		par.addSubBehaviour(new AnimalGetOldBehaviour(this));
		par.addSubBehaviour(new AnimalReproductionBehaviour(this));
		
		seq = new SequentialBehaviour(this);
		seq.addSubBehaviour(new AnimalGetMatureBehaviour(this));
		seq.addSubBehaviour(par);
	

		par = new ParallelBehaviour(this, ParallelBehaviour.WHEN_ANY);
		par.addSubBehaviour(new EnergyLossBehaviour(this));
		par.addSubBehaviour(new MoveBehaviour(this));
		par.addSubBehaviour(new BeEatenBehaviour(this));
		par.addSubBehaviour(seq);


		seq = new SequentialBehaviour(this);
		seq.addSubBehaviour(par);
		seq.addSubBehaviour(new DeadBehaviour(this));

		this.addBehaviour(seq);
	}
}
