package agents;

import enums.Phases;
import enums.Types;
import estructures.MapPosition;

import java.util.Random;

import managers.MapManager;
import behaviours.*;
import jade.core.Agent;
import jade.core.behaviours.*;

public abstract class LivingAgent extends Agent {

	protected static final long serialVersionUID = 3493143872944535412L;

	protected Types type;
	protected Phases phase;

	protected MapPosition position;

	protected int watchingDistance;
	protected int borningDistance;
	protected int reproductionEnergy;
	protected int eatingEnergy;

	protected int energy;
	protected int maxEnergy;

	public Types getType() {
		return type;
	}

	public void setType(Types type) {
		this.type = type;
	}

	public Phases getPhase() {
		return phase;
	}

	public void setPhase(Phases phase) {
		this.phase = phase;
	}

	public int getEatingEnergy() {
		return eatingEnergy;
	}

	public int getMaxEnergy() {
		return maxEnergy;
	}

	public int getReproductionEnergy() {
		return reproductionEnergy;
	}

	public int getWatchingDistance() {
		return watchingDistance;
	}

	public int getBorningDistance() {
		return borningDistance;
	}

	public int getEnergy() {
		return energy;
	}

	public void addEnergy(int energy) {
		this.energy += energy;
		if (this.energy > maxEnergy)
			this.energy = maxEnergy;
	}

	public boolean enoughEnergyToReproduce() {
		return energy >= reproductionEnergy;
	}

	public void addEatingEnergy() {
		addEnergy(eatingEnergy);
	}

	public void removeReproductionEnergy() {
		removeEnergy(reproductionEnergy);
	}

	public void removeEnergy(int energy) {
		this.energy -= energy;
	}

	public MapPosition getMapPosition() {
		return position;
	}

	protected void setup() {
		try {
			position = (MapPosition) this.getArguments()[0];
		} catch (Exception e) {
			e.printStackTrace();
			position = MapManager.getRandomFreePosition();
		}

		MapManager.initialize(this);
		MapManager.addToMap(this);

	}

	protected void takeDown() {
		MapManager.removeFromMap(this);
	}
}
