package behaviours;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import enums.Phases;
import enums.Types;
import estructures.MapPosition;
import managers.MapManager;
import agents.LivingAgent;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

public class MoveBehaviour extends TickerBehaviour {

	private LivingAgent food;
	private LivingAgent partner;

	public MoveBehaviour(LivingAgent a) {
		super(a, 200);
	}

	@Override
	protected void onTick() {

		List<LivingAgent> foodTargets = new ArrayList<LivingAgent>();
		List<LivingAgent> partnerTargets = new ArrayList<LivingAgent>();

		LivingAgent myLivingAgent = (LivingAgent) myAgent;

		if (food == null) {
			switch (((LivingAgent) myAgent).getType()) {
			case CARNIVORE:
				foodTargets.addAll(MapManager.getNearbyAgents(Types.HERBIVORE,
						Phases.YOUNG, ((LivingAgent) myAgent).getMapPosition(),
						myLivingAgent.getWatchingDistance()));
				foodTargets.addAll(MapManager.getNearbyAgents(Types.HERBIVORE,
						Phases.ADULT, ((LivingAgent) myAgent).getMapPosition(),
						myLivingAgent.getWatchingDistance()));
				break;
			case HERBIVORE:
				foodTargets.addAll(MapManager.getNearbyAgents(Types.PLANT,
						Phases.YOUNG, ((LivingAgent) myAgent).getMapPosition(),
						myLivingAgent.getWatchingDistance()));
				foodTargets.addAll(MapManager.getNearbyAgents(Types.PLANT,
						Phases.ADULT, ((LivingAgent) myAgent).getMapPosition(),
						myLivingAgent.getWatchingDistance()));
				break;
			}

			if (foodTargets.isEmpty()) {
				food = null;
			} else if (food == null) {
				int foodIndex = (int) (new Random().nextFloat() * foodTargets
						.size());
				food = foodTargets.get(foodIndex);
			}

		}

		if (partner == null) {
			switch (((LivingAgent) myAgent).getType()) {
			case CARNIVORE:
				partnerTargets.addAll(MapManager.getNearbyAgents(
						Types.CARNIVORE, Phases.ADULT,
						((LivingAgent) myAgent).getMapPosition(),
						myLivingAgent.getWatchingDistance()));
				break;
			case HERBIVORE:
				partnerTargets.addAll(MapManager.getNearbyAgents(
						Types.HERBIVORE, Phases.ADULT,
						((LivingAgent) myAgent).getMapPosition(),
						myLivingAgent.getWatchingDistance()));
				break;
			}

			if (partnerTargets.isEmpty()) {
				partner = null;
			} else if (partner == null
					&& myLivingAgent.getPhase() == Phases.ADULT) {
				int partnerIndex = (int) (new Random().nextFloat() * partnerTargets
						.size());
				partner = partnerTargets.get(partnerIndex);
			}
		}
		
		
		
		
		
		

		if (partner != null && myLivingAgent.getPhase() == Phases.ADULT
				&& myLivingAgent.enoughEnergyToReproduce()) {
			if (MapManager.isAdyacent(partner.getMapPosition(),
					myLivingAgent.getMapPosition())) {

				requestReproduction();

			} else {
				//System.out.println(myAgent.getLocalName()+ ": me muevo hacia la pareja "+ partner.getLocalName());
				MapPosition pos= MapManager.getCloserFreeAdyacentPosition(myLivingAgent.getMapPosition(),partner.getMapPosition());
				if(pos!=null){	
				MapManager.moveInMap(myLivingAgent,pos);
				}else{
					partner=null;
				}
				
			}

			if (food != null && food.getPhase() != Phases.DEAD
					&& MapManager.isAdyacent(food.getMapPosition(),
							myLivingAgent.getMapPosition())) {				
				eat();
			}
			
			food=null;

		} else if (food != null && food.getPhase() != Phases.DEAD) {
			partner = null;
			if (MapManager.isAdyacent(food.getMapPosition(),
					myLivingAgent.getMapPosition())) {
				eat();
			} else {
				//System.out.println(myAgent.getLocalName()+ ": me muevo hacia la comida " + food.getLocalName());
				MapPosition pos= MapManager.getCloserFreeAdyacentPosition(myLivingAgent.getMapPosition(),food.getMapPosition());
				if(pos!=null){	
				MapManager.moveInMap(myLivingAgent,pos);
				}else{
					food=null;
				}
				
			}
		} else {
			partner = null;
			//System.out.println(myAgent.getLocalName()+ ": me muevo aleatoriamente");
			MapPosition pos= MapManager.getRandomNearbyFreePosition(myLivingAgent.getMapPosition(), 1);
			if(pos!=null){			
				MapManager.moveInMap(myLivingAgent,pos);
			}
		}

	}

	
	
	
	private void eat() {
		LivingAgent myLivingAgent = (LivingAgent) myAgent;

		ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
		msg.setContent("Eaten");
		msg.addReceiver(food.getAID());
		myLivingAgent.send(msg);

		myLivingAgent.addEatingEnergy();

		System.out.println(myAgent.getLocalName() + ": Pareces apetitoso "
				+ food.getLocalName() + ". Creo que voy a comerte");
		food = null;
	}

	
	
	
	private void requestReproduction() {
		LivingAgent myLivingAgent = (LivingAgent) myAgent;

		ACLMessage msg = new ACLMessage(ACLMessage.PROPOSE);
		msg.setContent("Reproduce");
		msg.addReceiver(partner.getAID());
		myLivingAgent.send(msg);

		System.out.println(myAgent.getLocalName()
				+ ": Quieres pasar un buen rato " + partner.getLocalName()
				+ "?");
		partner = null;
	}

}
